;
;	
;	
;	SMS REMOTE CONTROL, FOR ERICSSON T10 GSM MOBILE PHONE
;
;	This circuit can handle 8 devices (ON/OFF) by sending SMS message with a mobile phone. 
;
;   21.08.2006  V2.1  --- Removed <response SMS> because the cell phone provider is infor us if SMS is delivered (free of charge! )
;                     ... Added a reset command. You can send a SMS like this form: 110r1000 or 110R1000. 
;                     ... 1 = relay ON
;                     ... 0 = Relay OFF
;                     ... r or R = Reset. Put the current relay in OFF state for 2 seconds and then, in ON state until next change, by a new SMS. 
;
;	03.07.2004	V2.00 --- I have rewrite the source code and i add more features.
;
;	04.10.2003  V1.01 --- I have change some routines to be more workable
;
;	30-10-2002	V1.0  --- I start it	
;
;
;
;
;	- Plug and play function added. There is no need to power on the mobile phone...
;	  ...beffore the AVR. Just connect the devices (AVR & T10s) and turn them ON (power-ON).
;		 The system communication will start immediately.
;	- Response SMS to sender, added. Now the sender of the SMS command will be informed with return-SMS...
;     ... from the AVR, if SMS command is executed.
;
;	writed with AVR studio 4.07
;
;	by Serasidis Vasilis 
;
;	Home: http://www.serasidis.gr
;	email: info@serasidis.gr 
;		   avrsite@yahoo.gr
;	
;
;	Target: AT90S2313 or ATtiny2313 at 8 MHz crystal
;	PD0= RxD
;	PD1= TxD
;	PB0-PB7= 8 bit output. You can drive with this port, 8 relays, 1 to each pin.
;
;	Send SMS message to T10, like this format: 11100101. 
;	That its mean, arm the relays No 1,3,6,7,8 and disarm the relays 2,4 and 5.
;	After Version 2.1 you can send the command r or R for reset (for more see above).
;      New format is like 11r00101 or 11R00101. Its mean, arm relays 1,3,7,8 disarm the 2,4,5
;      and disarm for 2 seconds the 6th relay and then arm it until next change.
;	
;	



.include "tn2313def.inc"	;Define chip particulars (ATtiny2313)


.equ	BAUD		=51 	;9600 bps at 8.00 MHz.
.equ	RamStart	=96
.equ	SMSC		=100	;RAM address for SMS Service Center 
.equ	TPDA		=118	;RAM address for Sender's number 
.equ	TimeDate	=134	;RAM address for Time and date of received SMS
.equ	SMScommand	=150	;RAM address for encoded SMS command (11001010)
.equ	ASCII_HEX 	=185	;RAM address for ASCII to HEX conversion
.equ	decodedSMS 	=200	;RAM address for decoded SMS
.equ	TxD			=3		;Transmition pin is PD3 (Data from AVR to PC computer)

.def    byte1a   =r2
.def    byte2a   =r3
.def    byte3a   =r4
.def    byte4a   =r5
.def    byte5a   =r6
.def    byte6a   =r7
.def    byte7a   =r8
.def    byte8a   =r9

.def	temp	=R16		;temporary storage register
.def	interrupt=R17
.def	Txbyte	=R18		;Data to be transmitted
.def	Rxbyte	=R19		;Received data
.def	temp2	=R20
.def	temp3	=R21
.def	temp4	=R22
.def	temp5	=R23
.def	bitcnt	=R23		;bit counter
.def	temp6	=R24
.def	counter	=R25		;


.def	delay1	=R26
.def	delay2	=R27
.def	YLramadr=R28
.def	YHramadr=R29
.def	delay3	=R29
.def	ZLramadr=R30
.def	ZHramadr=R31
.cseg
;========================================================================


.org	0
	rjmp	reset

.org	OVF0addr			; Timer/Counter0 Overflow

	in 		r0,SREG
	inc		interrupt
	cpi		interrupt,80
	breq	rst2
	out 	SREG,r0
	reti


rst2:
	pop		temp

	rjmp	timer_overflow

;*******************************************************************

cmd:	.db	"AT+CPMS=",'"',"ME",'"',0x0d,0	;Select the phone memory ("ME")
cmd2:	.db	"AT+CMGR=1" ,0x0d, 0 			;read the received message(Memory possition 1)
cmd3:	.db	"AT+CMGD=1" ,0x0d, 0			;delete the received message (Memory possition 1)

;==============================================================VVVV
;   *********** IS NOT USED AFTER Version 2.1 ***********     
;cmd4:	.db "AT+CMGS=125",0x0d,0	;Send SMS command
;msg1:	.db "000100",0
;msg2:	.db	"7FC377BB1D769341",0	; "Command "
;
;msg3:	.db   "A0F41C54C697C7"		; " is exec"
;		.db   "757A991C02A1C6"      ; "uted! (c"
;		.db   "29190C460389F3"		; ")2004 by"
;		.db	  "A069591E9EA7C9"		; " Serasid"
;		.db	  "E939C81A9EA7D9"		; "is Vasil"
;		.db	  "E939084DA7C375"		; "is http:"
;		.db	  "AFD7FD7E77CDCB"		; "//www.se"
;		.db	  "F2F03C4D4ECF5D"		; "rasidis."
;		.db	  "673988556E87D3"		; "gr ,emai"
;		.db	  "EC3928ED36BF01"		; "ls info@"
;		.db	  "F3B23C3C4F93D3"		; "serasidi"
;		.db	  "73D7590E6281C2" 		; "s.gr , a"
;		.db   "76F93C4D2F03F2"		; "vrsite@y"
;		.db   "61F4FBED3ACB01",0 	; "ahoo.gr"			      
;==============================================================^^^^								   	

reset:

	ldi		temp,RAMEND
	out		SPL,temp			;Init Stack Pointer


	ldi		temp,0b11111111
	out		DDRB,temp
	ldi		temp,0b11111010
	out		DDRD,temp
	ldi		temp,0b00000101
	out		portD,temp

	ldi		temp,(1<<CS02)+(0<<CS01)+(1<<CS00)
	out		TCCR0,temp			; Set clock to count on CK/1024
	ldi		temp,(1<<TOIE0)
	out		TIMSK,temp			; enable timer 0 overflow interrupt

	ldi		temp,BAUD
	out		UBRR,temp			;Set baud rate generator


;===========   Set UART, as Transmiter   ======================

	ldi		temp,0b00011000
	out		UCR,temp			;Enable UART Tx and Rx 
;==============================================================

;	sbic	pinD,2			;pressed PD2?
;	rjmp	timer_overflow
;	ser		temp
;	out		portb,temp

;==============================================================
;		Send AT+CPMS="ME" command
;==============================================================
timer_overflow:

;    ldi     YL,ramstart
;    ldi     counter,40
;    clr     temp
;clear_ram_piece:
;    st      Y+,temp
;    dec     counter
;    brne    clear_ram_piece

	rcall	long_delay
	rcall	Set_Mem

;==============================================================
;		Send AT+CMGR=1 command
;==============================================================

Start: 
	clr		interrupt
	rcall	long_delay

load:
	rcall	Read_Mem1
	rcall	delay				;delay some time beffore read the T10s

	ldi		YL,RamStart			;It is the first byte of RAM
	clr		YHramadr
	clr		ZHramadr
	ldi		temp2,5				;write 5 bytes from UART to ram

again:

	rcall	receive	
	cpi		rxbyte,'+'			;Ignore 1st plus. It is the echo of AT command that i sent.
	brne	again				;

again3:
	rcall	receive	
	cpi		rxbyte,'+'			;wait for start byte '+' ( +CMGR:0,,... )
	brne	again3				;

uart2ram:
	rcall	receive				;call receive routine. Read the <CMS E> or <CMGR:> text.
	st		Y+,rxbyte			;store data to RAM and increase RAM address
	dec		temp2				;Is temp2 zero ?
	brne	uart2ram			;if not, read next byte.

;--------------------------------------------------------

	ldi		YL,98				;Memory potition for character 'G' in ram 
	ld		txbyte,Y+			;load the contain of selected potition
	cpi		txbyte,'G'			;Look if its 'G' letter ( G from CMGR:)
	brne	timer_overflow		;If not, go to send again the AT commands
	ld		txbyte,Y+			;load the contain of selected potition
	cpi		txbyte,'R'			;Look if its 'R' letter ( R from CMGR:)
	brne	timer_overflow		;If not, go to send again the AT commands

read_message:	

	rcall	long_delay			;delay some time
	rcall	Read_Mem1			;Send the Read_memory1 command again.
load2:

	ldi		YLramadr,RamStart	;The first byte of RAM
	ldi		temp2,68			;use 68 bytes of ram

again2:
	rcall	receive				;Receive byte
	cpi		rxbyte,'0'			; it is '0' ? ( 0 is the first character of the message data 07910396249100002...)
	brne	again2				; if not, read next byte.

uart2ram2:
	st		Y+,rxbyte			;store data to RAM
	rcall	receive				;call receive routine
	dec		temp2
	brne	uart2ram2			;if not done, loop more



	rcall	nextline			;\
	ldi		txbyte,'+'			; \
	rcall	PC_transmit			;  \
	ldi		YLramadr,SMSC 		;  / Print the service center number
	ldi		temp3,6				; /
	rcall	read				;/

	rcall	nextline			;\
	ldi		txbyte,'+'			; \
	rcall	PC_transmit			;  \
	ldi		YLramadr,TPDA 		;  / Print the sender`s number 
	ldi		temp3,6				; /
	rcall	read				;/

	rcall	nextline			;\
	ldi		YLramadr,TimeDate 	; \
	ldi		temp3,1				;  \
	rcall	read				;   \	
	ldi		txbyte,'-'			;    \ 	
	rcall	PC_transmit			;     \	
	ldi		temp3,1				;      \ 
	rcall	read				;		|
	ldi		txbyte,'-'			;		|
	rcall	PC_transmit			;		|
	ldi		temp3,1				;		|
	rcall	read				;		\
	ldi		txbyte,' '			;		/ Print the date and the time of the received message.
	rcall	PC_transmit			;		|
	ldi		temp3,1				;		|
	rcall	read				;		|
	ldi		txbyte,':'			;		|
	rcall	PC_transmit			;      /
	ldi		temp3,1				;     / 
	rcall	read				;    /
	ldi		txbyte,':'			;   /
	rcall	PC_transmit			;  /
	ldi		temp3,1				; /
	rcall	read				;/

;===================================================================================
; Here its the routine for convertion the ASCII to HEX and stored to RAM address 185
;===================================================================================

	ldi		ZLramadr,ASCII_HEX
	clr		ZHramadr 		    ;
	ldi		temp3,7				;7 bits length
	clr		YHramadr
	ldi		YLramadr,SMScommand	;First byte of the message in the RAM (message like this 11001101)


septet:	


	ld		txbyte,Y+			;Load byte from RAM
	cpi		txbyte,0x41			;compare it with 0x41 ("a" hex)
	brlo	number				; if it is lower, it is a number, not letter. So, go to number address.

	ldi		temp4,9				;
	add		txbyte,temp4		;add 9 to txbyte
	cbr		txbyte,0b11110000	;clear bits 7,6,5,4 of txbyte
	swap	txbyte				; swap nibles
	mov		temp,txbyte			;copy txbyte to temp
	rjmp	sec_ascii			;go to sec_ascii (second ASCII)

number:
	cbr		txbyte,0b11110000	;If 1st byte is number, clear bits 7,6,5,4 of txbyte
	swap	txbyte				;swap nibles
	mov		temp,txbyte			;copy txbyte to temp


sec_ascii:

	ld		txbyte,Y+			;Load 2nd byte to txbyte
	cpi		txbyte,0x41			; compare it with 0x41
	brlo	number2				;if it is lower than 0x41, go to "number2"

	ldi		temp4,9				;
	add		txbyte,temp4		;add 9 to txbyte
	cbr		txbyte,0b11110000	;clear bits 7,6,5,4 of txbytes
	add		txbyte,temp			;add  temp to txbyte
	rjmp	store_in_ram		;go to "store_in_ram"

number2:
	cbr		txbyte,0b11110000	
	add		txbyte,temp

store_in_ram:

	st		Z+,txbyte			;Save the txbyte to RAM
	dec		temp3				;decrease temp3
	brne	septet				;if its not '0', go to "septet"


;==============================================================
; convert the HEX values to septets (make 7 bit of PDU, 8 bit).
;==============================================================

	rcall	nextline        	;
	ldi		YLramadr,ASCII_HEX	;

    clr     byte1a
    ld      byte2a,Y+
    ld      byte3a,Y+
    ld      byte4a,Y+
    ld      byte5a,Y+
    ld      byte6a,Y+
    ld      byte7a,Y+
    ld      byte8a,Y+

    clc                 ;clear carry flag  
    rcall   shift_byte8
    clc                 ;clear carry flag  
    rcall   shift_byte7
    clc                 ;clear carry flag  
    rcall   shift_byte6
    clc                 ;clear carry flag  
    rcall   shift_byte5
    clc                 ;clear carry flag  
    rcall   shift_byte4
    clc                 ;clear carry flag  
    rcall   shift_byte3
    clc                 ;clear carry flag  
    rcall   shift_byte2
    clc                 ;clear carry flag  
    rcall   shift_byte1

    ldi		YLramadr,decodedSMS ;RAM address (200) for decoded SMS (8 bytes)

    mov     txbyte,byte1a
    st      Y+,txbyte           ;store data in RAM
    mov     txbyte,byte2a
    st      Y+,txbyte           ;store data in RAM
    mov     txbyte,byte3a
    st      Y+,txbyte           ;store data in RAM
    mov     txbyte,byte4a
    st      Y+,txbyte           ;store data in RAM
    mov     txbyte,byte5a
    st      Y+,txbyte           ;store data in RAM
    mov     txbyte,byte6a
    st      Y+,txbyte           ;store data in RAM
    mov     txbyte,byte7a
    st      Y+,txbyte           ;store data in RAM
    mov     txbyte,byte8a
    st      Y+,txbyte           ;store data in RAM

;--------------------- Check command for PB0 -------------------------------
    ldi		YLramadr,decodedSMS	;RAM address (200) for decoded SMS (8 bytes)

PB0_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB0_OFF
    sbi     PortB,0 	;set PB0 pin
PB0_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB0_reset
    cbi     PortB,0 	;Clear PB0 pin
PB0_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB0_Capital_reset
    cbi     PortB,0 	;clear PB0 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,0 	;set PB0 pin

PB0_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	PB1_ON
    cbi     PortB,0 	;clear PB0 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,0 	;set PB0 pin
;--------------------- Check command for PB1 -------------------------------
PB1_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB1_OFF
    sbi     PortB,1 	;set PB1 pin
PB1_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB1_reset
    cbi     PortB,1 	;Clear PB1 pin
PB1_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB1_Capital_reset
    cbi     PortB,1 	;clear PB1 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,1 	;set PB1 pin

PB1_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	PB2_ON
    cbi     PortB,1 	;clear PB1 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,1 	;set PB1 pin
;--------------------- Check command for PB2 -------------------------------
PB2_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB2_OFF
    sbi     PortB,2 	;set PB2 pin
PB2_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB2_reset
    cbi     PortB,2 	;Clear PB2 pin
PB2_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB2_Capital_reset
    cbi     PortB,2 	;clear PB2 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,2 	;set PB2 pin

PB2_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	PB3_ON
    cbi     PortB,2 	;clear PB2 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,2 	;set PB2 pin
;--------------------- Check command for PB3 -------------------------------
PB3_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB3_OFF
    sbi     PortB,3 	;set PB3 pin
PB3_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB3_reset
    cbi     PortB,3 	;Clear PB3 pin
PB3_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB3_Capital_reset
    cbi     PortB,3 	;clear PB3 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,3 	;set PB3 pin

PB3_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	PB4_ON
    cbi     PortB,3 	;clear PB3 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,3 	;set PB3 pin

;--------------------- Check command for PB4 -------------------------------
PB4_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB4_OFF
    sbi     PortB,4 	;set PB4 pin
PB4_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB4_reset
    cbi     PortB,4 	;Clear PB4 pin
PB4_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB4_Capital_reset
    cbi     PortB,4 	;clear PB4 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,4 	;set PB4 pin

PB4_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	PB5_ON
    cbi     PortB,4 	;clear PB4 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,4 	;set PB4 pin
;--------------------- Check command for PB5 -------------------------------
PB5_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB5_OFF
    sbi     PortB,5 	;set PB5 pin
PB5_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB5_reset
    cbi     PortB,5 	;Clear PB5 pin
PB5_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB5_Capital_reset
    cbi     PortB,5 	;clear PB5 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,5 	;set PB5 pin

PB5_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	PB6_ON
    cbi     PortB,5 	;clear PB5 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,5 	;set PB5 pin
;--------------------- Check command for PB6 -------------------------------
PB6_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB6_OFF
    sbi     PortB,6 	;set PB6 pin
PB6_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB6_reset
    cbi     PortB,6 	;Clear PB6 pin
PB6_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB6_Capital_reset
    cbi     PortB,6 	;clear PB6 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,6 	;set PB6 pin

PB6_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	PB7_ON
    cbi     PortB,6 	;clear PB6 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,6 	;set PB6 pin
;--------------------- Check command for PB7 -------------------------------
PB7_ON:
    rcall   long_delay2

	ld		txbyte,Y+
	rcall	PC_transmit

	cpi		txbyte,'1'  ;check if received <relay ON> command.
	brne	PB7_OFF
    sbi     PortB,7 	;set PB7 pin
PB7_OFF:
	cpi		txbyte,'0'  ;check if received <relay OFF> command.
	brne	PB7_reset
    cbi     PortB,7 	;Clear PB7 pin
PB7_reset:
	cpi		txbyte,'r'  ;check if received <relay Reset> command (small 'r').
	brne	PB7_Capital_reset
    cbi     PortB,7 	;clear PB7 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,7 	;set PB7 pin

PB7_Capital_reset:
	cpi		txbyte,'R'  ;check if received <relay Reset> command (capital 'R').
	brne	Clear_entire_RAM
    cbi     PortB,7 	;clear PB7 pin
    rcall   long_delay
    rcall   long_delay
    sbi     PortB,7 	;set PB7 pin

	rcall	nextline    ;

;===========================================================
;	Clear entire RAM, 128 bytes
;===========================================================
Clear_entire_RAM:

    rcall	delete_mem1         ;Delete SMS from phone memory, pottition 1.
    rcall   delay

	rcall	delay
	ldi		YLramadr,RamStart
	ser		temp
clr_ram:
	st		Y+,temp
	cpi		YL,220
	brne	clr_ram

	rjmp	timer_overflow

;-----------------------------------------------------------
shift_byte8:
    ror     byte8a
shift_byte7:
    ror     byte7a
shift_byte6:
    ror     byte6a
shift_byte5:
    ror     byte5a
shift_byte4:
    ror     byte4a
shift_byte3:
    ror     byte3a
shift_byte2:
    ror     byte2a
shift_byte1:
    ror     byte1a
    ret                 
;===========================================================
;	Read 2 ASCII characters, swap them and send them to T10s
;===========================================================

read:
	inc		YLramadr
	ld		txbyte,Y
	rcall	PC_transmit
	dec		YLramadr
	ld		txbyte,Y
	rcall	PC_transmit
	inc		YLramadr
	inc		YLramadr
	dec		temp3
	brne	read

	ret	

;=============================================================
;		<Change line> and <Enter> ASCII commands
;=============================================================
nextline:
	ldi		txbyte,0x0d
	rcall	PC_transmit
	ldi		txbyte,0x0a
	rcall	PC_transmit

	ret
;***************************************************************************
;*	Transmition routine TxD (Data from AVR are going to PC computer).
;***************************************************************************

PC_transmit:
    
    push    txbyte      ;Backup txbyte register in RAM
	ldi		bitcnt,10	;1+8+sb (sb is # of stop bits)
	com		Txbyte		;Invert everything

	sec					;Start bit

putchar0:
	brcc	putchar1	;If carry set
	cbi		PORTD,TxD	;    send a '0'
	rjmp	putchar2	;else	

putchar1:
	sbi		PORTD,TxD	;    send a '1'
	nop

putchar2:
	rcall	UART_delay	;One bit delay
	rcall	UART_delay

	lsr		Txbyte		;Get next bit
	dec		bitcnt		;If not all bit sent
	brne	putchar0	;send next
    pop     txbyte      ;restore txbyte register from RAM

	ret					;return
;***************************************************************************
;*
;* "UART_delay"
;*
;* This delay subroutine generates the required delay between the bits when
;* transmitting and receiving bytes. The total execution time is set by the
;* constant "b":
;*
;*	3xb + 7 cycles (including rcall and ret)
;*
;* Number of words	:4 including return
;* Low registers used	:None
;* High registers used	:1 (temp)
;* Pointers used	:None
;*
;***************************************************************************
; Some b values: 	
;
; 1 MHz crystal:
;   9600 bps - b=14
;  19200 bps - b=5
;  28800 bps - b=2
;
; 2 MHz crystal:
;  19200 bps - b=14
;  28800 bps - b=8
;  57600 bps - b=2

; 4 MHz crystal:
;   9600 bps - b=62
;  19200 bps - b=31
;  28800 bps - b=19
;  57600 bps - b=8
; 115200 bps - b=2

.equ	b=62		;19200 bps @ 8 MHz crystal


UART_delay:
	ldi	    temp,b
UART_delay1:
	dec	    temp
	brne	UART_delay1
	ret

;***************************************************************************
; Transmiting routine (The data from AVR are going to T10s)         
;***************************************************************************
b_transmit:	
	sbis	USR,UDRE	;is UART transmitter ready?
	rjmp	b_transmit
	out		UDR,txbyte	;sent out char
	ret

;*************************************************************************
; Receiving routine (The data from T10s are going to AVR) 
;*************************************************************************
receive:
	sei		;Set Enable Interrupts
rec_loop:
	sbis	USR,RXC
	rjmp	rec_loop
	cli		;Clear Interrupts
	in		Rxbyte,UDR
	clr		interrupt
	ret


;*******************************************************************
;                   Stop and Delay routines
;*******************************************************************
long_delay2:
    ldi     delay3,20
    rjmp    waitmore2

long_delay:
	ldi		delay3,40
waitmore2:
	rcall	delay
	dec		delay3
	brne	waitmore2
	ret	

delay:
	ldi		delay2,0xff
waitsome:
	ldi		delay1,0xff			

waitmore:
	dec		delay1
	brne	waitmore
	dec		delay2
	brne	waitsome
	
	ret


;*******************************************************************
Set_Mem:
	ldi		zl,low(cmd*2)	;set pointer to
	ldi		zh,high(cmd*2)	;the text
	rjmp	send_command

Read_Mem1:
	ldi		zl,low(cmd2*2)	;set pointer to
	ldi		zh,high(cmd2*2)	;the text
	rjmp	send_command

Delete_Mem1:
	ldi		zl,low(cmd3*2)	;set pointer to
	ldi		zh,high(cmd3*2)	;the text
	;rjmp	send_command

send_command:
	lpm
	tst		r0				;check if its the last character (the zero)
	breq	return			;if yes, return.
	mov		txbyte,r0
	rcall	b_transmit		;
	adiw	zl,1			;increase zl
	rjmp	send_command	;read next character from flash memory

return:
	ret


